/*
   Copyright (C) Andrew Tridgell 2002
   Copyright (C) Stefan Westerfeld 2009   

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* the windows execute code is taken from ccache-win32 */

#include <stdio.h>
#include <string>
#include <windows.h>

static char *argvtos(char **argv)
{
    int i, len;
    char *ptr, *str, *aptr;

    for (i = 0, len = 0; argv[i]; i++) {
	len += 2 * strlen(argv[i]) + 3; /* upper bound: each character escaped */
    }

    str = ptr = (char *)malloc(len + 1);
    if (str == NULL)
	return NULL;

    for (i = 0; argv[i]; i++) {
	*ptr++ = '"';
	for (aptr = argv[i]; *aptr; aptr++)
	{
	    if (*aptr == '"')
		*ptr++ = '\\';
	    *ptr++ = *aptr;
	}
	*ptr++ = '"';
	*ptr++ = ' ';
    }
    *ptr = 0;

    return str;
}

/*
  execute a compiler backend, capturing all output to the given paths
  the full path to the compiler to run is in argv[0]
*/
int execute (char **argv)
{
    PROCESS_INFORMATION pinfo; 
    STARTUPINFO sinfo;
    BOOL ret; 
    DWORD exitcode;
    char *args;

    ZeroMemory(&pinfo, sizeof(PROCESS_INFORMATION));
    ZeroMemory(&sinfo, sizeof(STARTUPINFO));

    sinfo.cb = sizeof(STARTUPINFO); 

    args = argvtos(argv);

    ret = CreateProcessA(argv[0], args, NULL, NULL, TRUE, 0, NULL, NULL,
	    &sinfo, &pinfo);

    free(args);

    if (ret == 0)
	return -1;

    WaitForSingleObject(pinfo.hProcess, INFINITE);
    GetExitCodeProcess(pinfo.hProcess, &exitcode);
    CloseHandle(pinfo.hProcess);
    CloseHandle(pinfo.hThread);

    return exitcode;
}

/*
  find an executable by name in $PATH
*/
char *find_executable(const char *name)
{
    DWORD ret;
    char namebuf[MAX_PATH];

    ret = SearchPathA(NULL, name, ".exe", sizeof(namebuf), namebuf, NULL);
    if (ret != 0) {
	return strdup (namebuf);
    }

    return NULL;
}

using std::string;

int
main (int argc, char **argv)
{
    if (argc < 3)
    {
	fprintf (stderr, "usage: %s <slash_pattern> <program> <args>", argv[0]);
	return 1;
    }
    string slash_pattern = argv[1];
    argv[2] = find_executable (argv[2]);

    char **new_argv = (char **) malloc (sizeof (char *) * argc);
    for (int i = 2; i < argc; i++)
    {
	string arg = argv[i];
	string xarg;
	for (int j = 0; j < arg.size(); j++)
	{
	    if (arg.compare (j, slash_pattern.size(), slash_pattern) == 0)
	    {
		xarg += "/";
		j += slash_pattern.size() - 1;
	    }
	    else
	    {
		xarg += arg[j];
	    }
	}
	new_argv[i - 2] = strdup (xarg.c_str());
    }
    new_argv[argc - 2] = NULL;
    return execute (new_argv);
}
